
/*
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published 
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ex6.h"

int main(int argc,char **argv)
{
  int listenfd,connfd;
  id x,y,s,q,privkey,pubkey,ppubkey,eprivkey,epubkey,t,u;
  struct sockaddr_in servaddr;

  setup();

  q = [field cardinality];
  privkey = [BigInt str:serv_privkey];
  printf("Private key : \n");[privkey printLine];

  printf("Public key : \n");
  x = [field embed:[BigInt str:serv_pubkeyx]];
  y = [field embed:[BigInt str:serv_pubkeyy]];
  pubkey = [[PointOfCurve X:x Y:y] curve:curve];
  [pubkey printLine];

  printf("Public key of client : \n");
  x = [field embed:[BigInt str:cli_pubkeyx]];
  y = [field embed:[BigInt str:cli_pubkeyy]];
  ppubkey = [[PointOfCurve X:x Y:y] curve:curve];
  [ppubkey printLine];

  eprivkey = [bigintrandom([order numDigits]) modulo:order];
  printf("Ephemeral Private key : \n");[eprivkey printLine];

  printf("Ephemeral Public key (this may take a while to compute given the private key, and the reverse should be even harder to compute): \n");
  printdots++;epubkey = [basepoint multiply:eprivkey];printdots=0;printf("\n");
  [epubkey printLine];
  
  if ((listenfd = socket(AF_INET,SOCK_STREAM,0)) < 0) {
    perror("socket");exit(0);
  }

  memset(&servaddr,0,sizeof(struct sockaddr_in));
  servaddr.sin_family = AF_INET;
  servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  servaddr.sin_port = htons(PORT);

  if (bind(listenfd, (struct sockaddr *)&servaddr, sizeof(servaddr)) < 0) {
    perror("bind");exit(0);
  }

  if (listen(listenfd,LISTENQ) < 0) {
    perror("listen");exit(0);
  }

  printf("\nReady to accept connections ...\n");
  
  while (1) {
    id k;
    connfd = accept(listenfd,(struct sockaddr *)NULL,NULL); 
    if (connfd < 0) { perror("accept");continue; }

    k = [pubkey receiveFrom:connfd];
    [epubkey sendOn:connfd];
    printf("Ephemeral public key of peer : \n");
    [k printLine];

    s = [[[epubkey X] multiply:[pubkey X]] asBigInt];
    t = [privkey add:[s multiply:eprivkey]];
    t = [t modulo:order];

    printf("Shared Menezes-Qu-Vanstone secret : \n");
    printdots++;
    s = [[[k X] multiply:[ppubkey X]] asBigInt];
    u = [ppubkey add:[k multiply:[s modulo:order]]];
    u = [u multiply:t];
    printdots = 0; printf("\n");

    [u printLine];
  }
}

